import { ActionRowBuilder, ButtonBuilder, ButtonStyle, EmbedBuilder } from "discord.js";
import Inventory from "../database/entities/app/Inventory";
import { CoreClient } from "../client/client";
import SeriesMetadata, { CardMetadata } from "../contracts/SeriesMetadata";
import EmbedColours from "../constants/EmbedColours";

export default class InventoryHelper {
    public static async GenerateInventoryPage(userid: string, page: number): Promise<{ embed: EmbedBuilder, row: ActionRowBuilder<ButtonBuilder> }> {
        const cardsPerPage = 10;

        const inventory = await Inventory.FetchAllByUserId(userid);

        const allSeries: SeriesMetadata[] = [];

        for (let item of inventory) {
            const series = CoreClient.Cards.find(x => x.cards.find(y => y.id == item.CardNumber));

            if (!series || allSeries.includes(series)) {
                continue;
            }

            allSeries.push(series);
        }

        const seriesToDisplay: SeriesMetadata[] = [];

        for (let series of allSeries.sort((a, b) => a.id - b.id)) {
            const seriesCards = series.cards.filter(x => inventory.find(y => y.CardNumber == x.id));

            series.cards = seriesCards;

            seriesToDisplay.push(series);
        }

        const pages: {
            id: number,
            name: string,
            cards: {
                cardNumber: string,
                name: string,
                quantity: number,
            }[],
            seriesSubpage: number,
        }[] = [];

        for (let series of seriesToDisplay) {
            for (let i = 0; i < series.cards.length; i+= cardsPerPage) {
                const cards = series.cards.slice(i, i + cardsPerPage);
                const pageCards: {
                    cardNumber: string,
                    name: string,
                    quantity: number,
                }[] = [];

                for (let card of cards) {
                    const item = inventory.find(x => x.CardNumber == card.id);

                    if (!item) {
                        continue;
                    }

                    pageCards.push({
                        cardNumber: card.id,
                        name: card.name,
                        quantity: item.Quantity,
                    });
                }

                pages.push({
                    id: series.id,
                    name: series.name,
                    cards: pageCards,
                    seriesSubpage: i / cardsPerPage,
                });
            }
        }

        const currentPage = pages[page];

        if (!currentPage) {
            console.error("Unable to find page");
            return Promise.reject("Unable to find page");
        }

        const embedDescription: string[] = [];

        for (let card of currentPage.cards) {
            embedDescription.push(`[${card.cardNumber}] ${card.name} (${card.quantity})`);
        }

        const embed = new EmbedBuilder()
            .setTitle(`${currentPage.name} (${currentPage.seriesSubpage + 1})`)
            .setDescription(embedDescription.join('\n'))
            .setFooter({ text: `Page ${page} of ${pages.length}` })
            .setColor(EmbedColours.Ok);

        const row = new ActionRowBuilder<ButtonBuilder>()
            .addComponents(
                new ButtonBuilder()
                    .setCustomId(`inventory ${userid} ${page - 1}`)
                    .setLabel("Previous")
                    .setStyle(ButtonStyle.Primary)
                    .setDisabled(page == 0),
                new ButtonBuilder()
                    .setCustomId(`inventory ${userid} ${page + 1}`)
                    .setLabel("Next")
                    .setStyle(ButtonStyle.Primary)
                    .setDisabled(page == pages.length));

        return { embed, row };
    }
}